( cd usr/share/blackbox/nls ; rm -rf de )
( cd usr/share/blackbox/nls ; ln -sf de_DE de )
( cd usr/share/blackbox/nls ; rm -rf en )
( cd usr/share/blackbox/nls ; ln -sf C en )
( cd usr/share/blackbox/nls ; rm -rf es )
( cd usr/share/blackbox/nls ; ln -sf es_ES es )
( cd usr/share/blackbox/nls ; rm -rf fr )
( cd usr/share/blackbox/nls ; ln -sf fr_FR fr )
( cd usr/share/blackbox/nls ; rm -rf it )
( cd usr/share/blackbox/nls ; ln -sf it_IT it )
( cd usr/share/blackbox/nls ; rm -rf nl )
( cd usr/share/blackbox/nls ; ln -sf nl_NL nl )
( cd usr/share/blackbox/nls ; rm -rf POSIX )
( cd usr/share/blackbox/nls ; ln -sf C POSIX )
( cd usr/share/blackbox/nls ; rm -rf en_GB )
( cd usr/share/blackbox/nls ; ln -sf C en_GB )
( cd usr/share/blackbox/nls ; rm -rf en_US )
( cd usr/share/blackbox/nls ; ln -sf C en_US )
( cd usr/share/blackbox/nls ; rm -rf es_MX )
( cd usr/share/blackbox/nls ; ln -sf es_ES es_MX )
( cd usr/share/blackbox/nls ; rm -rf US_ASCII )
( cd usr/share/blackbox/nls ; ln -sf C US_ASCII )
